<?php
echo <<<'START'
********************************************************************************
   __________________ __________   ________       _____  __________ ___________
  /   _____/\_   ___ \\______   \  \_____  \     /  _  \ \______   \\__    ___/
  \_____  \ /    \  \/ |       _/   /  ____/    /  /_\  \ |       _/  |    |   
  /        \\     \____|    |   \  /       \   /    |    \|    |   \  |    |   
 /_______  / \______  /|____|_  /  \_______ \  \____|__  /|____|_  /  |____|   
         \/         \/        \/           \/          \/        \/            
        
           Image Converter from ZX Spectrum .SCR to C64 ArtStudio .ART
                               by TheTom/SAMAR           v 1.0

********************************************************************************
START;

// Website: http://tomseditor.com/gallery
// Github : https://github.com/TomsEditor
// License: MIT

@mkdir('art');

foreach (glob('scr/*.scr') AS $inName)
{
	$outName = 'art/' . pathinfo($inName, PATHINFO_FILENAME) . '.art';

	$res = scr2art($inName, $outName, $error);
	if (!$res)
	{
		echo basename($inName) . ': ' . $error . "\r\n";
	}
}

function scr2art($inName, $outName, &$error)
{
	$file = file_get_contents($inName);

	if (strlen($file) != 6912)
	{
		$error = 'Not a valid .SCR file';
		return 0;
	}

	$pixels  = substr($file, 0, 6144);
	$palette = substr($file, 6144);

	$zxBright = array(0x000000,0x0000FF,0xFF0000,0xFF00FF,0x00FF00,0x00FFFF,0xFFFF00,0xFFFFFF);
	$zxDark = array(0x000000,0x0000CD,0xCD0000,0xCD00CD,0x00CD00,0x00CDCD,0xCDCD00,0xCDCDCD);
		
	$im = imagecreatetruecolor(256, 192);

	$p = 0;

	for ($k=0; $k<3; $k++)
	{
		for ($j=0; $j<8; $j++)
		{
			for ($y=0; $y<ceil(64/8); $y++)
			{
				for ($x=0; $x<ceil(256/8); $x++)
				{
					$byte = ord($pixels[$p]);
					for ($i=0; $i<8; $i++)
					{
						$index = $byte >> (7-$i) & 1;
						imagesetpixel($im, 8*$x + $i, 64*$k + 8*$y + $j, $index);
					}
					$p++;
				}
			}
		}
	}

	$p = 0;

	for ($y=0; $y<ceil(192/8); $y++)
	{
		for ($x=0; $x<ceil(256/8); $x++)
		{
			$color = ord($palette[$p++]);

			for ($j=0; $j<8; $j++)
			{
				for ($i=0; $i<8; $i++)
				{
					$foreground = imagecolorat($im, 8*$x + $i, 8*$y + $j);

					if ($foreground)
					{
						$index = $color & 7;
					}
					else
					{
						$index = ($color >> 3) & 7;
					}
					$bright = ($color >> 6) & 1;

					if ($bright)
					{
						$rgb = $zxBright[$index];
					}
					else
					{
						$rgb = $zxDark[$index];
					}

					imagesetpixel($im, 8*$x + $i, 8*$y + $j, $rgb);
				}
			}
		}
	}

	$im2 = imagecreatetruecolor(320, 200);
	imagecopy($im2, $im, 32,8, 0,0, 256, 192);
	imagedestroy($im);

	$cell = imagecreatetruecolor(8, 8);

	$spec = array(0=>0,205=>6,255=>6,13434880=>2,16711680=>2,13435085=>4,16711935=>4,
		52480=>5,65280=>13,52685=>3,65535=>3,13487360=>7,16776960=>7,13487565=>15,16777215=>1);

	$out = chr(0).chr(32);

	$palette = '';

	for ($y=0; $y<25; $y++)
	{
		for ($x=0; $x<40; $x++)	
		{
			imagecopy($cell, $im2, 0,0, $x*8, $y*8, 8, 8);

			$uniq = array();

			for ($j=0; $j<8; $j++)
			{
				for ($i=0; $i<8; $i++)
				{
					$rgb = imagecolorat($cell, $i, $j);
					$uniq[$rgb] = 1; 
				}
			}

			$uniq = array_keys($uniq);
			$count = count($uniq);

			if ($count > 2)
			{
				$error = 'More than 2 colors per cell';
				return 0;
			}
			if ($count == 1)
			{
				$uniq[] = 0;
			}

			for ($j=0; $j<8; $j++)
			{
				$byte = 0;
				for ($i=0; $i<8; $i++)
				{
					$rgb = imagecolorat($cell, $i, $j);
					$index = ($rgb === $uniq[0]) ? 0 : 1;

					$byte = $byte + ($index << (7-$i));
				}
				$out .= chr($byte);
			}

			if (isset($spec[$uniq[0]]) and isset($spec[$uniq[1]]) )
			{
				$color1 = $spec[$uniq[0]];
				$color2 = $spec[$uniq[1]];
			}
			else
			{
				$error = 'Image needs to use Pepto palette';
				return 0;
			}

			$palette .= chr($color1 + ($color2 << 4));
		}
	}

	$out .= $palette.chr(0);

	imagedestroy($cell);
	imagedestroy($im2);

	file_put_contents($outName, $out);
	return 1;
}